import smbus2
import time

# Device address and register definitions
GT911_I2C_ADDR = 0x5D        # Adjust based on actual hardware
GT_CFG_START_REG = 0x8047    # Configuration data start address
GT_CHECKSUM_REG = 0x80FF     # Checksum register
GT_FLASH_SAVE_REG = 0x8100   # Flash save control register

# Example configuration table (replace with actual data)
GT911_CFG_TBL = [
    0xFF, 0x40, 0x01, 0xE0, 0x01, 0x05, 0x81, 0x00, 0x08, 0xFF,
    0x1E, 0x0F, 0x5A, 0x3C, 0x03, 0x05, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x89, 0x20, 0x06,
    0x37, 0x35, 0x43, 0x06, 0x00, 0x00, 0x01, 0xB9, 0x03, 0x1C,
    0x63, 0x00, 0x00, 0x00, 0x00, 0x03, 0x64, 0x32, 0x00, 0x00,
    0x00, 0x28, 0x64, 0x94, 0xC5, 0x02, 0x07, 0x00, 0x00, 0x04,
    0x99, 0x2C, 0x00, 0x84, 0x34, 0x00, 0x71, 0x3F, 0x00, 0x5E,
    0x4C, 0x00, 0x4F, 0x5B, 0x00, 0x4F, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x0C, 0x0A, 0x08, 0x06, 0x04, 0x02, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x0A, 0x0C, 0x0F, 0x10, 0x08, 0x06, 0x04, 0x02,
    0x00, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00]

def calculate_checksum(config_table):
    """Calculate Checksum: Complement of the Sum of All Bytes"""
    checksum = sum(config_table) & 0xFF
    return (~checksum + 1) & 0xFF

def write_register(bus, register, data):
    """Write to Register (Supports Single or Multiple Bytes)"""
    if not isinstance(data, list):
        data = [data]
    max_data_per_write = 30
    for i in range(0, len(data), max_data_per_write):
        chunk = data[i:i+max_data_per_write]
        reg_high = (register >> 8) & 0xFF
        reg_low = (register + i) & 0xFF
        try:
            bus.write_i2c_block_data(GT911_I2C_ADDR, reg_low, chunk)
            print(f"Write successful: Register 0x{(reg_high << 8) | reg_low:04X}, Data length {len(chunk)}")
            time.sleep(0.01)
        except Exception as e:
            print(f"Write failed: Register 0x{(reg_high << 8) | reg_low:04X}, Error: {e}")
            raise

def gt911_send_cfg(bus, config_table, mode=0):
    """Send configuration to GT911"""
    if len(config_table) != 184:
        raise ValueError("The configuration data must be 184 bytes.")
    
    # Write configuration in chunks
    max_data_per_write = 30
    for i in range(0, 184, max_data_per_write):
        chunk = config_table[i:i+max_data_per_write]
        write_register(bus, GT_CFG_START_REG + i, chunk)
    
    # Write checksum and mode
    checksum = calculate_checksum(config_table)
    write_register(bus, GT_CHECKSUM_REG, [checksum, mode])
    
    # Save to Flash
    if mode == 1:
        write_register(bus, GT_FLASH_SAVE_REG, 0x01)
        print("Configuration has been saved to Flash...")
    else:
        print("Configuration not saved to Flash...")

def main():
    with smbus2.SMBus(1) as bus:
        try:
            print("Start sending configuration...")
            gt911_send_cfg(bus, GT911_CFG_TBL, mode=0)
            print("Configuration sent successfully")
        except Exception as e:
            print(f"Configuration sending failed: {e}")

if __name__ == "__main__":
    main()